@@ -4,7 +4,7 @@ use warnings;
 use strict;
 
 use base 'Exporter';
-use HOP::Stream qw/drop tail head node is_node/;
+use HOP::Stream qw/drop tail head/;
 
 our %N;
 
@@ -20,8 +20,6 @@ our @EXPORT_OK = qw(
   list_of
   list_values_of
   lookfor
-  lookahead
-  neg_lookahead
   match
   nothing
   null_list
@@ -31,7 +29,6 @@ our @EXPORT_OK = qw(
   rlist_of
   rlist_values_of
   star
-  plus
   T
   test
 );
@@ -46,16 +43,16 @@ HOP::Parser - "Higher Order Perl" Parser
 
 =head1 VERSION
 
-Version 0.02_05
+Version 0.01
 
 =cut
 
-our $VERSION = '0.02_05';
+our $VERSION = '0.01';
 
 =head1 SYNOPSIS
 
   use HOP::Parser qw/:all/;
-
+  
   # assemble a bunch of parsers according to a grammar
 
 =head1 DESCRIPTION
@@ -65,11 +62,8 @@ by Mark Jason Dominus.
 
 This module implements recursive-descent parsers by allowing programmers to
 build a bunch of smaller parsers to represent grammar elements and assemble
-them into a full parser. Pages 376 to 415 of the first and second editions
-of HOP should be enough to get you up to speed :)
-
-The PDF for the second edition can be downloaded from MJD's site: 
-L<http://hop.perl.plover.com/book/pdf/HigherOrderPerl.pdf>.
+them into a full parser. Pages 376 to 415 of the first edition of HOP should
+be enough to get you up to speed :)
 
 Please note that this module should be considered B<ALPHA> code.  While
 everything works fairly well, the documentation is incomplete and some of the
@@ -103,10 +97,6 @@ functions could stand to be better named (C<rlist_of>, for example).
 
 =item * match
 
-=item * lookahead
-
-=item * neg_lookahead
-
 =item * nothing
 
 =item * null_list
@@ -123,8 +113,6 @@ functions could stand to be better named (C<rlist_of>, for example).
 
 =item * star
 
-=item * plus
-
 =item * T
 
 =item * test
@@ -215,8 +203,6 @@ used instead.
 
 =cut
 
-my $is_node = sub { is_node($_[0]) || ref $_[0] eq 'ARRAY' };
-
 sub lookfor {
     my $wanted = shift;
     my $value  = shift || sub { $_[0][1] };
@@ -225,7 +211,6 @@ sub lookfor {
     $wanted = [$wanted] unless ref $wanted;
     my $parser = parser {
         my $input = shift;
-        $input = node(@$input) if ref($input) eq 'ARRAY';
         unless ( defined $input ) {
             die [ 'TOKEN', $input, $wanted ];
         }
@@ -246,8 +231,9 @@ sub lookfor {
         # Otherwise, the AoA stream might just return an aref for
         # the tail instead of an AoA.  This breaks things
         my $tail = tail($input);
-        $tail = [$tail]
-            if $is_node->($tail) and not $is_node->($tail->[0]);
+        if ( 'ARRAY' eq ref $tail && 'ARRAY' ne ref $tail->[0] ) {
+            $tail = [$tail];
+        }
         return ( $wanted_value, $tail );
     };
     $N{$parser} = "[@$wanted]";
@@ -284,59 +270,10 @@ sub parser (&) { $_[0] }
 
 ##############################################################################
 
-=head2 lookahead
-
-  my $parser = lookahead( $label );
-  $parser = lookahead( $parser );
-
-This function takes a parser argument or list of arguments supported by
-C<lookfor()> and returns a parser that will return true if the parser matches,
-but does not actually change the stream. This is so that you can write parsers
-that match something and then look ahead to see if they match the next thing,
-without actualy consuming that next thing. This is akin to a zero width
-positive look-ahead in a regular expression.
-
-=cut
-
-sub lookahead {
-    my $p = ref $_[0] eq 'CODE' ? shift : lookfor @_;
-    parser {
-        my $input = shift or return;
-        $p->($input);
-        return (undef, $input);
-    },
-}
-
-##############################################################################
-
-=head2 neg_lookahead
-
-  my $parser = neg_lookahead( $label );
-  $parser = neg_lookahead( $parser );
-
-This function returns a parser that returns true if it looks ahead and does
-not find a match for the specified parser. That is, it's akin to a zero width
-negative look-ahead in a regular expression. The supported arguments are the
-same as for C<lookahead()>.
-
-=cut
-
-sub neg_lookahead {
-    my $p = ref $_[0] eq 'CODE' ? shift : lookfor @_;
-    parser {
-        my $input = shift or return;
-        my @ret = eval { $p->($input) };
-        die [ 'TOKEN', $input, $p ] if @ret;
-        return (undef, $input);
-    },
-}
-
-##############################################################################
-
 =head2 concatenate
 
   my $parser = concatenate(@parsers);
-  my ($values, $remainder) = $parser->($stream);
+  ok ($values, $remainder) = $parser->($stream);
 
 This function takes a list of parsers and returns a new parser. The new parser
 succeeds if all parsers passed to C<concatenate> succeed sequentially.
@@ -424,7 +361,15 @@ sub list_of {
 
     return T(
         concatenate( $element, star( concatenate( $separator, $element ) ) ),
-        sub {[ $_[0], $_[1] ? map { @$_ } @{ $_[1] } : () ] },
+        sub {
+            my @matches = shift;
+            if ( my $tail = shift ) {
+                foreach my $match (@$tail) {
+                    push @matches, @$match;
+                }
+            }
+            return \@matches;
+        }
     );
 }
 
@@ -572,24 +517,22 @@ sub null_list {
   my ($parsed, $remainder) = $parser->($stream);
 
 This parser always succeeds and matches zero or more instances of
-C<$another_parser>. It parallels the regular expression C<*> quantifier. If it
-matches zero, it returns the same results as C<null_list>. Otherwise, it
-returns and array ref of the matched values and the remainder of the stream.
+C<$another_parser>.  If it matches zero, it returns the same results as
+C<null_list>.  Otherwise, it returns and array ref of the matched values and
+the remainder of the stream.
 
 =cut
 
-my $star_plus_t = sub {
-    my ( $first, $rest ) = @_;
-    [ $first, @$rest ];
-};
-
 sub star {
     my $p = shift;
     my $p_star;
     $p_star = alternate(
         T(
             concatenate( $p, parser { $p_star->(@_) } ),
-            $star_plus_t,
+            sub {
+                my ( $first, $rest ) = @_;
+                [ $first, @$rest ];
+            }
         ),
         \&null_list
     );
@@ -597,42 +540,19 @@ sub star {
 
 ##############################################################################
 
-=head2 plus
-
-  my $parser = plus($another_parser);
-  my ($parsed, $remainder) = $parser->($stream);
-
-This parser succeeds when it matches one or more instances of
-C<$another_parser>. It parallels the regular expression C<+> quantifier. If it
-matches one or more, it returns and array ref of the matched values and the
-remainder of the stream.
-
-=cut
-
-sub plus {
-    my $p = shift;
-    T(
-        concatenate( $p, star($p) ),
-        $star_plus_t,
-    );
-}
-
-##############################################################################
-
 =head2 optional
 
  my $parser = optional($another_parser);
- my ($parser, $remainder) = $parser->(stream);
+ my ($parser, $remainder) = $parser->(stream); 
 
-This parser matches 0 or 1 of the given parser item. It parallels the regular
-expression C<?> quantifier.
+This parser matches 0 or 1 of the given parser item.
 
 =cut
 
 sub optional {
     my $parser = shift;
     return alternate (
-        $parser,
+        T($parser, sub { [ shift ] }),
         \&null_list,
     );
 }
@@ -772,11 +692,6 @@ All Software (code listings) presented in the book can be found on the
 companion website for the book (http://perl.plover.com/hop/) and is
 subject to the License agreements below.
 
-=head1 LATEST VERSION
-
-You can download the latest versions of these modules at
-L<http://github.com/Ovid/hop/>.  Feel free to fork and make changes.
-
 =head1 ELSEVIER SOFTWARE LICENSE AGREEMENT
 
 Please read the following agreement carefully before using this Software. This
@@ -847,7 +762,7 @@ identifying the original source of the Software: "from Higher-Order Perl by
 Mark Dominus, published by Morgan Kaufmann Publishers, Copyright 2005 by
 Elsevier Inc".
 
-=item 2 Disclaimer of Warranty.
+=item 2 Disclaimer of Warranty. 
 
 We make no warranties at all. The Software is transferred to you on an "as is"
 basis. You use the Software at your own peril. You assume all risk of loss for
@@ -858,7 +773,7 @@ any patent, copyright, or proprietary right. All other warranties, expressed
 or implied, including, without limitation, any warranty of merchantability or
 fitness for a particular purpose are hereby excluded.
 
-=item 3 Limitation of Liability.
+=item 3 Limitation of Liability. 
 
 We will have no liability for special, incidental, or consequential damages
 even if advised of the possibility of such damages. We will not be liable for
